/* volumeBase.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"

#include "volumeBase.h"
#include "fileVolume.h"
#include "dataCell.h"
#include "queryResult.h"
#include "database.h"
#include "expand_variables.h"

#include <stdio.h>

VolumeBase::
VolumeBase(const Dataset &data) : _data(data) {
  _volume_id = data[TI_volumes].get_primary_key();
  assert(_volume_id >= 0);
}

VolumeBase::
~VolumeBase() {
}

void VolumeBase::
setup() {
  _definition = _data[TI_volumes][VFI_definition].get_string();
  _bucket = _data[TI_volumes][VFI_current_bucket].get_int();
  _bucket_size = _data[TI_volumes][VFI_bucket_size].get_int();
  _bname_unexpanded = _data[TI_volumes][VFI_bucket_name].get_string();
  expand_bname();
}

VolumeBase *VolumeBase::
make_volume(const Dataset &data) {
  if (data[TI_volumes].get_primary_key() < 0) {
    // No Volume table.
    return NULL;
  } else {
    return new FileVolume(data);
  }
}

string VolumeBase::
get_description() const {
  return _data[TI_volumes][VFI_description].get_string();
}

string VolumeBase::
get_definition() const {
  return _data[TI_volumes][VFI_definition].get_string();
}

int VolumeBase::
get_volume_id() const {
  return _data[TI_volumes].get_primary_key();
}

void VolumeBase::
read_db() {
  // Re-fetch the parameters describing the current volume from the
  // database.  We want to be absolutely sure we're getting the
  // current information.

  QueryResult qr = music_db->sql_command
    ("SELECT definition, current_bucket, bucket_size, bucket_name"
     " FROM volumes"
     " WHERE volume_id=" + DataCell(_volume_id));

  assert(qr.get_num_rows() > 0);
  
  _definition = qr.get_cell(0, 0).get_string();
  _bucket = qr.get_cell(0, 1).get_int();
  _bucket_size = qr.get_cell(0, 2).get_int();
  _bname_unexpanded = qr.get_cell(0, 3).get_string();

  expand_bname();
}

void VolumeBase::
expand_bname() {
  char bucket_str[128];
  sprintf(bucket_str, "%03d", _bucket);

  VariableDef vars;
  vars["bucket"] = bucket_str;

  _bname = expand_variables(_bname_unexpanded, vars);

  if (_bname.empty()) {
    _dirname = _definition;
  } else {
    _dirname = _definition + "/" + _bname;
  }
}
