/* iso8859_to_ascii.C

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#include "system.h"
#include "iso8859_to_ascii.h"

// The ISO 8859-1 character set.  This list was extracted from the
// iso_8859_1(7) man page.

#define NO_BREAK_SPACE                                  0xa0
#define INVERTED_EXCLAMATION_MARK                       0xa1
#define CENT_SIGN                                       0xa2
#define POUND_SIGN                                      0xa3
#define CURRENCY_SIGN                                   0xa4
#define YEN_SIGN                                        0xa5
#define BROKEN_BAR                                      0xa6
#define SECTION_SIGN                                    0xa7
#define DIAERESIS                                       0xa8
#define COPYRIGHT_SIGN                                  0xa9
#define FEMININE_ORDINAL_INDICATOR                      0xaa
#define LEFT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK       0xab
#define NOT_SIGN                                        0xac
#define SOFT_HYPHEN                                     0xad
#define REGISTERED_SIGN                                 0xae
#define MACRON                                          0xaf
#define DEGREE_SIGN                                     0xb0
#define PLUS_MINUS_SIGN                                 0xb1
#define SUPERSCRIPT_TWO                                 0xb2
#define SUPERSCRIPT_THREE                               0xb3
#define ACUTE_ACCENT                                    0xb4
#define MICRO_SIGN                                      0xb5
#define PILCROW_SIGN                                    0xb6
#define MIDDLE_DOT                                      0xb7
#define CEDILLA                                         0xb8
#define SUPERSCRIPT_ONE                                 0xb9
#define MASCULINE_ORDINAL_INDICATOR                     0xba
#define RIGHT_POINTING_DOUBLE_ANGLE_QUOTATION_MARK      0xbb
#define VULGAR_FRACTION_ONE_QUARTER                     0xbc
#define VULGAR_FRACTION_ONE_HALF                        0xbd
#define VULGAR_FRACTION_THREE_QUARTERS                  0xbe
#define INVERTED_QUESTION_MARK                          0xbf
#define LATIN_CAPITAL_LETTER_A_WITH_GRAVE               0xc0
#define LATIN_CAPITAL_LETTER_A_WITH_ACUTE               0xc1
#define LATIN_CAPITAL_LETTER_A_WITH_CIRCUMFLEX          0xc2
#define LATIN_CAPITAL_LETTER_A_WITH_TILDE               0xc3
#define LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS           0xc4
#define LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE          0xc5
#define LATIN_CAPITAL_LIGATURE_AE                       0xc6
#define LATIN_CAPITAL_LETTER_C_WITH_CEDILLA             0xc7
#define LATIN_CAPITAL_LETTER_E_WITH_GRAVE               0xc8
#define LATIN_CAPITAL_LETTER_E_WITH_ACUTE               0xc9
#define LATIN_CAPITAL_LETTER_E_WITH_CIRCUMFLEX          0xca
#define LATIN_CAPITAL_LETTER_E_WITH_DIAERESIS           0xcb
#define LATIN_CAPITAL_LETTER_I_WITH_GRAVE               0xcc
#define LATIN_CAPITAL_LETTER_I_WITH_ACUTE               0xcd
#define LATIN_CAPITAL_LETTER_I_WITH_CIRCUMFLEX          0xce
#define LATIN_CAPITAL_LETTER_I_WITH_DIAERESIS           0xcf
#define LATIN_CAPITAL_LETTER_ETH                        0xd0
#define LATIN_CAPITAL_LETTER_N_WITH_TILDE               0xd1
#define LATIN_CAPITAL_LETTER_O_WITH_GRAVE               0xd2
#define LATIN_CAPITAL_LETTER_O_WITH_ACUTE               0xd3
#define LATIN_CAPITAL_LETTER_O_WITH_CIRCUMFLEX          0xd4
#define LATIN_CAPITAL_LETTER_O_WITH_TILDE               0xd5
#define LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS           0xd6
#define MULTIPLICATION_SIGN                             0xd7
#define LATIN_CAPITAL_LETTER_O_WITH_STROKE              0xd8
#define LATIN_CAPITAL_LETTER_U_WITH_GRAVE               0xd9
#define LATIN_CAPITAL_LETTER_U_WITH_ACUTE               0xda
#define LATIN_CAPITAL_LETTER_U_WITH_CIRCUMFLEX          0xdb
#define LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS           0xdc
#define LATIN_CAPITAL_LETTER_Y_WITH_ACUTE               0xdd
#define LATIN_CAPITAL_LETTER_THORN                      0xde
#define LATIN_SMALL_LETTER_SHARP_S                      0xdf
#define LATIN_SMALL_LETTER_A_WITH_GRAVE                 0xe0
#define LATIN_SMALL_LETTER_A_WITH_ACUTE                 0xe1
#define LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX            0xe2
#define LATIN_SMALL_LETTER_A_WITH_TILDE                 0xe3
#define LATIN_SMALL_LETTER_A_WITH_DIAERESIS             0xe4
#define LATIN_SMALL_LETTER_A_WITH_RING_ABOVE            0xe5
#define LATIN_SMALL_LIGATURE_AE                         0xe6
#define LATIN_SMALL_LETTER_C_WITH_CEDILLA               0xe7
#define LATIN_SMALL_LETTER_E_WITH_GRAVE                 0xe8
#define LATIN_SMALL_LETTER_E_WITH_ACUTE                 0xe9
#define LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX            0xea
#define LATIN_SMALL_LETTER_E_WITH_DIAERESIS             0xeb
#define LATIN_SMALL_LETTER_I_WITH_GRAVE                 0xec
#define LATIN_SMALL_LETTER_I_WITH_ACUTE                 0xed
#define LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX            0xee
#define LATIN_SMALL_LETTER_I_WITH_DIAERESIS             0xef
#define LATIN_SMALL_LETTER_ETH                          0xf0
#define LATIN_SMALL_LETTER_N_WITH_TILDE                 0xf1
#define LATIN_SMALL_LETTER_O_WITH_GRAVE                 0xf2
#define LATIN_SMALL_LETTER_O_WITH_ACUTE                 0xf3
#define LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX            0xf4
#define LATIN_SMALL_LETTER_O_WITH_TILDE                 0xf5
#define LATIN_SMALL_LETTER_O_WITH_DIAERESIS             0xf6
#define DIVISION_SIGN                                   0xf7
#define LATIN_SMALL_LETTER_O_WITH_STROKE                0xf8
#define LATIN_SMALL_LETTER_U_WITH_GRAVE                 0xf9
#define LATIN_SMALL_LETTER_U_WITH_ACUTE                 0xfa
#define LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX            0xfb
#define LATIN_SMALL_LETTER_U_WITH_DIAERESIS             0xfc
#define LATIN_SMALL_LETTER_Y_WITH_ACUTE                 0xfd
#define LATIN_SMALL_LETTER_THORN                        0xfe
#define LATIN_SMALL_LETTER_Y_WITH_DIAERESIS             0xff

string
iso8859_to_ascii(const string &str) {
  string result;

  for (string::const_iterator pi = str.begin();
       pi != str.end(); 
       ++pi) {
    switch ((unsigned char)(*pi)) {
    case LATIN_CAPITAL_LETTER_A_WITH_GRAVE:
    case LATIN_CAPITAL_LETTER_A_WITH_ACUTE:
    case LATIN_CAPITAL_LETTER_A_WITH_CIRCUMFLEX:
    case LATIN_CAPITAL_LETTER_A_WITH_TILDE:
    case LATIN_CAPITAL_LETTER_A_WITH_DIAERESIS:
    case LATIN_CAPITAL_LETTER_A_WITH_RING_ABOVE:
      result += 'A';
      break;
      
    case LATIN_CAPITAL_LIGATURE_AE:
      result += "AE";
      break;
      
    case LATIN_CAPITAL_LETTER_C_WITH_CEDILLA:
      result += 'C';
      break;
      
    case LATIN_CAPITAL_LETTER_E_WITH_GRAVE:
    case LATIN_CAPITAL_LETTER_E_WITH_ACUTE:
    case LATIN_CAPITAL_LETTER_E_WITH_CIRCUMFLEX:
    case LATIN_CAPITAL_LETTER_E_WITH_DIAERESIS:
      result += 'E';
      break;
      
    case LATIN_CAPITAL_LETTER_I_WITH_GRAVE:
    case LATIN_CAPITAL_LETTER_I_WITH_ACUTE:
    case LATIN_CAPITAL_LETTER_I_WITH_CIRCUMFLEX:
    case LATIN_CAPITAL_LETTER_I_WITH_DIAERESIS:
      result += 'I';
      break;
      
    case LATIN_CAPITAL_LETTER_ETH:
      result += "ETH";
      break;
      
    case LATIN_CAPITAL_LETTER_N_WITH_TILDE:
      result += 'N';
      break;
      
    case LATIN_CAPITAL_LETTER_O_WITH_GRAVE:
    case LATIN_CAPITAL_LETTER_O_WITH_ACUTE:
    case LATIN_CAPITAL_LETTER_O_WITH_CIRCUMFLEX:
    case LATIN_CAPITAL_LETTER_O_WITH_TILDE:
    case LATIN_CAPITAL_LETTER_O_WITH_DIAERESIS:
    case LATIN_CAPITAL_LETTER_O_WITH_STROKE:
      result += 'O';
      break;
      
    case LATIN_CAPITAL_LETTER_U_WITH_GRAVE:
    case LATIN_CAPITAL_LETTER_U_WITH_ACUTE:
    case LATIN_CAPITAL_LETTER_U_WITH_CIRCUMFLEX:
    case LATIN_CAPITAL_LETTER_U_WITH_DIAERESIS:
      result += 'U';
      break;
      
    case LATIN_CAPITAL_LETTER_Y_WITH_ACUTE:
      result += 'Y';
      break;
      
    case LATIN_CAPITAL_LETTER_THORN:
      result += "TH";
      break;
      
    case LATIN_SMALL_LETTER_SHARP_S:
      result += "ss";
      break;
      
    case LATIN_SMALL_LETTER_A_WITH_GRAVE:
    case LATIN_SMALL_LETTER_A_WITH_ACUTE:
    case LATIN_SMALL_LETTER_A_WITH_CIRCUMFLEX:
    case LATIN_SMALL_LETTER_A_WITH_TILDE:
    case LATIN_SMALL_LETTER_A_WITH_DIAERESIS:
    case LATIN_SMALL_LETTER_A_WITH_RING_ABOVE:
      result += 'a';
      break;
      
    case LATIN_SMALL_LIGATURE_AE:
      result += "ae";
      break;
      
    case LATIN_SMALL_LETTER_C_WITH_CEDILLA:
      result += 'c';
      break;
      
    case LATIN_SMALL_LETTER_E_WITH_GRAVE:
    case LATIN_SMALL_LETTER_E_WITH_ACUTE:
    case LATIN_SMALL_LETTER_E_WITH_CIRCUMFLEX:
    case LATIN_SMALL_LETTER_E_WITH_DIAERESIS:
      result += 'e';
      break;
      
    case LATIN_SMALL_LETTER_I_WITH_GRAVE:
    case LATIN_SMALL_LETTER_I_WITH_ACUTE:
    case LATIN_SMALL_LETTER_I_WITH_CIRCUMFLEX:
    case LATIN_SMALL_LETTER_I_WITH_DIAERESIS:
      result += 'i';
      break;

    case LATIN_SMALL_LETTER_ETH:
      result += "eth";
      break;

    case LATIN_SMALL_LETTER_N_WITH_TILDE:
      result += 'n';
      break;

    case LATIN_SMALL_LETTER_O_WITH_GRAVE:
    case LATIN_SMALL_LETTER_O_WITH_ACUTE:
    case LATIN_SMALL_LETTER_O_WITH_CIRCUMFLEX:
    case LATIN_SMALL_LETTER_O_WITH_TILDE:
    case LATIN_SMALL_LETTER_O_WITH_DIAERESIS:
    case LATIN_SMALL_LETTER_O_WITH_STROKE:
      result += 'o';
      break;

    case LATIN_SMALL_LETTER_U_WITH_GRAVE:
    case LATIN_SMALL_LETTER_U_WITH_ACUTE:
    case LATIN_SMALL_LETTER_U_WITH_CIRCUMFLEX:
    case LATIN_SMALL_LETTER_U_WITH_DIAERESIS:
      result += 'u';
      break;

    case LATIN_SMALL_LETTER_Y_WITH_ACUTE:
    case LATIN_SMALL_LETTER_Y_WITH_DIAERESIS:
      result += 'y';
      break;

    case LATIN_SMALL_LETTER_THORN:
      result += "th";
      break;

    default:
      if (isprint(*pi)) {
	result += *pi;
      }
    }
  }

  return result;
}

  
