/* form.h

   Copyright (C) 1999 by David Rose.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

   To contact the original author of this program, write to:
           David Rose
           224 W. Dryden St Apt. 418
	   Glendale, CA 91202
	   USA

	   drose@lafn.org
   */

#ifndef FORM_H
#define FORM_H

#include "dataset.h"
#include "formWidget.h"
#include "tableFieldKey.h"
#include "expand_variables.h"

#include <gtk--.h>
#include <vector>
#include <map>

class FormDef;

// This defines a group of widgets, defined by the user, that should
// appear on a dialog.

class Form : public Gtk_Signal_Base {
public:
  Form();
  ~Form();

  // A typecast operator to allow us to pack the form as if it were
  // itself a widget.
  operator Gtk_Widget & ();

  // The Form gets a reference to the Dataset, which it modifies
  // directly when update() is called.
  void layout_form(int form_id, Dataset &data);
  void show();

  // This writes the widget values into each of the associated cells
  // in the Dataset.
  void update();

  // This fills up a VariableDef object suitable for passing to
  // expand_variables() with each of the named values on the form.
  void get_variables(VariableDef &vars) const;

  // This returns the value the user set for the ratio slider
  // corresponding to the indicated field, or 0.0 if the user did not
  // set such a slider.
  double get_ratio_slider(TableID table_id, int field_id) const;

  // This is used by FormWidget::Instance to set the value of each
  // ratio slider when update() is called.
  void set_ratio_slider(TableID table_id, int field_id, double value);

  const FormDef &get_def() const;

  typedef map<TableFieldKey, double> RatioSliderValues;
  RatioSliderValues _ratio_slider_values;

private:
  typedef vector<FormWidget::Instance> Instances;
  Instances _instances;

  Gtk_Table _table;
  Gtk_ScrolledWindow _sc_window;
  const FormDef *_def;
  Dataset *_data;
};

#endif
